import { defineConfig } from 'vitepress'

// https://vitepress.dev/reference/site-config
export default defineConfig({
  title: "wandoubaba的技术收藏",
  description: "wandoubaba是一位很老的PHP程序员，他不是大牛却很喜欢研究程序，随着时间一点一点的积累了一些些技术资料，包括PHP、Docker、Worker(Webman)、RabbitMQ、FreeSWITCH、GoLang、Kubernete(k8s)等等。",
  srcDir: 'src',
  ignoreDeadLinks: true,
  themeConfig: {
    // https://vitepress.dev/reference/default-theme-config
    logo: '/wandoubaba.svg',
    footer: {
      message: 'Released under the MIT License.',
      copyright: 'Copyright © 2019-present wandoubaba'
    },
    nav: [
      { text: '首页', link: '/' },
      { text: 'PHP', link: '/php/' },
      { text: 'Docker', link: '/docker/' },
      { text: 'Go', link: '/go/' },
      { text: '前端', link: '/frontend/' },
      { text: 'FreeSWITCH', link: '/freeswitch/' }
    ],
    sidebar: [
      { text: '前言', link: '/start' },
      {
        text: 'Docker',
        items: [
          { text: '不常用命令备忘', link: '/docker/command' },
          { text: '安装Docker（debian包管理器）', link: '/docker/install-apt' },
          { text: '安装Docker（二进制文件）', link: '/docker/install-binary' },
          { text: '配置国内镜像源', link: '/docker/mirrors' },
          { text: '制作Vue开发环境镜像', link: '/docker/vue' },
          { text: '制作集成ik中文分词器的elasticsearch镜像', link: '/docker/elasticsearch-ik' },
          { text: '制作支持stomp和延时队列的rabbitmq镜像', link: '/docker/rabbitmq-stomp' },
          { text: '部署单节点MinIO服务', link: '/docker/minio' },
          { text: '部署Posrgres服务', link: '/docker/postgres' },
          { text: '部署一个私有Gitea服务', link: '/docker/gitea' }
        ]
      },
      {
        text: 'PHP',
        items: [
        // 'docker-phpfpm74',
        // 'docker81',
        // 'nginx',
        // 'openresty',
        // 'php82'
          { text: 'monolog保存日志到mongodb', link: '/php/monolog-mongo' },
          { text: '制作7.4的FPM环境镜像', link: '/php/docker-phpfpm74' },
          { text: '基于PHP8.x制作workman环境镜像', link: '/php/docker-php8' },
          { text: '源码编译安装PHP8.2环境', link: '/php/php82' },
          { text: '为PHP7.4安装pgsql和pdo_pgsql扩展', link: '/php/php74-pgsql' },
          { text: 'PHP的hash()函数中不同algo算法的性能对比', link: '/php/hash-algo' },
          { text: 'PHP对二维数组做去重并取得重复项（10万+记录）', link: '/php/php-2darray-filter' }
        ]
      },
      {
        text: '运维',
        items: [
          { text: '用acme.sh自动部署和续签SSL证书', link: '/ops/acme' },
          { text: '源码编译安装Nginx', link: '/ops/nginx' },
          { text: '源码编译安装OpenResty', link: '/ops/openresty' },
          { text: 'Linux以二进制方式安装node环境', link: '/ops/node' }
        ]
      },
      {
        text: 'FreeSWITCH',
        items: [
          { text: '制作FreeSWITCH1.10.12镜像的Dockerfile', link: 'freeswitch/dockerfile-1_10_12' },
          { text: 'Debian12编译安装FreeSWITCH1.10', link: 'freeswitch/debian12' },
          { text: 'Debian11/12中注册FreeSWITCH为系统服务', link: 'freeswitch/debian-service' },
          { text: 'Ubuntu16.04上安装FreeSWITCH1.6', link: 'freeswitch/ubuntu1604' },
          { text: 'FreeSWITCH安装mod_unimrcp模块', link: 'freeswitch/mod_unimrcp' }
        ]
      }
    ],

    socialLinks: [
      { icon: 'github', link: 'https://git.wandoubaba.com/wandoubaba/docs.git' }
    ]
  }
})
